@@ -1,25 +1,26 @@
-# Copyrights 2001-2017 by [Mark Overmeer].
+# Copyrights 2001-2019 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use strict;
+# This code is part of distribution Mail-Box-POP3.  Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Mail::Box::POP3::Test;
 use vars '$VERSION';
-$VERSION = '3.003';
+$VERSION = '3.005';
 
 use base 'Exporter';
 
+use strict;
+use warnings;
+
 use Mail::Transport::POP3;
 
 use List::Util 'first';
 use File::Spec;
 
-
-our @EXPORT =
-  qw/
-     start_pop3_server start_pop3_client
-    /;
+our @EXPORT = qw/start_pop3_server start_pop3_client/;
 
 #
 # Start POP3 server for tests
