@@ -1,22 +1,27 @@
-# Copyrights 2001-2017 by [Mark Overmeer].
+# Copyrights 2001-2019 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
+# This code is part of distribution Mail-Box-POP3.  Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
+
+package Mail::Transport::POP3;
+use vars '$VERSION';
+$VERSION = '3.005';
+
+use base 'Mail::Transport::Receive';
 
 use strict;
 use warnings;
 
-package Mail::Transport::POP3;
-use vars '$VERSION';
-$VERSION = '3.002';
-
-use base 'Mail::Transport::Receive';
-
 use IO::Socket  ();
 use Socket      qw/$CRLF/;
-use Digest::MD5 ();
+use Digest::MD5 qw/md5_hex/;
 
 
+sub _OK($) { substr(shift // '', 0, 3) eq '+OK' }
+
 sub init($)
 {   my ($self, $args) = @_;
     $args->{via}    = 'pop3';
@@ -24,24 +29,25 @@ sub init($)
 
     $self->SUPER::init($args) or return;
 
-    $self->{MTP_auth} = $args->{authenticate} || 'AUTO';
-    $self->{MTP_ssl}  = $args->{use_ssl};
-    return unless $self->socket;   # establish connection
+    $self->{MTP_auth}   = $args->{authenticate} || 'AUTO';
+    $self->{MTP_ssl}    = $args->{use_ssl};
+    $self->socket or return;   # establish connection
 
     $self;
 }
 
 #------------------------------------------
 
+sub useSSL() { shift->{MTP_ssl} }
+
+#------------------------------------------
 
 sub ids(;@)
 {   my $self = shift;
-    return unless $self->socket;
+    $self->socket or return;
     wantarray ? @{$self->{MTP_n2uidl}} : $self->{MTP_n2uidl};
 }
 
-#------------------------------------------
-
 
 sub messages()
 {   my $self = shift;
@@ -53,7 +59,7 @@ sub messages()
 }
 
 
-sub folderSize() { shift->{MTP_total} }
+sub folderSize() { shift->{MTP_folder_size} }
 
 
 sub header($;$)
@@ -147,7 +153,7 @@ sub disconnect()
      MTP_fetched
     ) };
 
-    OK($quit);
+    _OK $quit;
 }
 
 
@@ -163,9 +169,10 @@ sub id2n($;$) { shift->{MTP_uidl2n}{shift()} }
 #------------------------------------------
 
 
-sub socket(;$)
+sub socket()
 {   my $self = shift;
 
+    # Do we (still) have a working connection which accepts commands?
     my $socket = $self->_connection;
     return $socket if defined $socket;
 
@@ -175,15 +182,14 @@ sub socket(;$)
         return;
     }
 
-    return unless $socket = $self->login;
-    return unless $self->status( $socket );
-
-# Save socket in the object and return it
-
+    # (Re-)establish the connection
+    $socket = $self->login or return;
+    $self->status($socket) or return;
     $self->{MTP_socket} = $socket;
 }
 
 
+
 sub send($$)
 {   my $self = shift;
     my $socket = shift;
@@ -202,18 +208,15 @@ sub send($$)
 
 
 sub sendList($$)
-{   my $self     = shift;
-    my $socket   = shift;
-    my $response = $self->send($socket, @_) or return;
-
-    return unless OK($response);
+{   my ($self, $socket) = (shift, shift);
+    my $response = $self->send($socket, @_);
+    $response && _OK $response or return;
 
     my @list;
-    local $_;
-    while(<$socket>)
-    {   last if m#^\.\r?\n#s;
-        s#^\.##;
-	push @list, $_;
+    while(my $line = <$socket>)
+    {   last if $line =~ m#^\.\r?\n#s;
+        $line =~ s#^\.##;
+        push @list, $line;
     }
 
     \@list;
@@ -222,19 +225,17 @@ sub sendList($$)
 sub DESTROY()
 {   my $self = shift;
     $self->SUPER::DESTROY;
-    $self->disconnect if $self->{MTP_socket}; # only do if not already done
+    $self->disconnect if $self->{MTP_socket}; # only when open
 }
 
-sub OK($;$) { substr(shift || '', 0, 3) eq '+OK' }
-
-sub _connection(;$)
+sub _connection()
 {   my $self = shift;
 
     my $socket = $self->{MTP_socket};
-    defined $socket or return undef;
+    defined $socket or return;
 
     # Check if we (still) got a connection
-    eval {print $socket "NOOP$CRLF"};
+    eval { print $socket "NOOP$CRLF" };
     if($@ || ! <$socket> )
     {   delete $self->{MTP_socket};
         return undef;
@@ -244,66 +245,63 @@ sub _connection(;$)
 }
 
 
+
 sub login(;$)
 {   my $self = shift;
 
-# Check if we can make a TCP/IP connection
+    # Check if we can make a connection
 
-    local $_; # make sure we don't spoil $_ for the outside world
-    my ($interval, $retries, $timeout) = $self->retry;
     my ($host, $port, $username, $password) = $self->remoteHost;
     unless($username && $password)
     {   $self->log(ERROR => "POP3 requires a username and password.");
         return;
     }
 
-    my $net    = $self->{MTP_ssl} ? 'IO::Socket::SSL' : 'IO::Socket::INET';
+    my $net    = $self->useSSL ? 'IO::Socket::SSL' : 'IO::Socket::INET';
     eval "require $net" or die $@;
 
-    my $socket = eval {$net->new("$host:$port")};
+    my $socket = eval { $net->new("$host:$port") };
     unless($socket)
     {   $self->log(ERROR => "Cannot connect to $host:$port for POP3: $!");
         return;
     }
 
-# Check if it looks like a POP server
+    # Check if it looks like a POP server
 
     my $connected;
     my $authenticate = $self->{MTP_auth};
-    my $welcome = <$socket>;
-    unless(OK($welcome))
+    my $welcome      = <$socket>;
+    unless(_OK $welcome)
     {   $self->log(ERROR =>
            "Server at $host:$port does not seem to be talking POP3.");
         return;
     }
 
-# Check APOP login if automatic or APOP specifically requested
-
+    # Check APOP login if automatic or APOP specifically requested
     if($authenticate eq 'AUTO' || $authenticate eq 'APOP')
     {   if($welcome =~ m#^\+OK .*(<\d+\.\d+\@[^>]+>)#)
-        {   my $md5 = Digest::MD5::md5_hex($1.$password);
+        {   my $md5      = md5_hex $1.$password;
             my $response = $self->send($socket, "APOP $username $md5$CRLF");
-            $connected = OK($response) if $response;
+            $connected   = _OK $response;
         }
     }
 
-# Check USER/PASS login if automatic and failed or LOGIN specifically requested
-
+    # Check USER/PASS login if automatic and failed or LOGIN specifically
+    # requested.
     unless($connected)
     {   if($authenticate eq 'AUTO' || $authenticate eq 'LOGIN')
         {   my $response = $self->send($socket, "USER $username$CRLF")
                or return;
 
-            if(OK($response))
-	    {   $response = $self->send($socket, "PASS $password$CRLF")
+            if(_OK $response)
+            {   my $response2 = $self->send($socket, "PASS $password$CRLF")
                    or return;
-                $connected = OK($response);
+                $connected = _OK $response2;
             }
         }
     }
 
-# If we're still not connected now, we have an error
-
+    # If we're still not connected now, we have an error
     unless($connected)
     {   $self->log(ERROR => $authenticate eq 'AUTO' ?
          "Could not authenticate using any login method" :
@@ -314,39 +312,37 @@ sub login(;$)
     $socket;
 }
 
-#------------------------------------------
 
 
 sub status($;$)
-{   my ($self,$socket) = @_;
+{   my ($self, $socket) = @_;
 
-# Check if we can do a STAT
+    # Check if we can do a STAT
 
     my $stat = $self->send($socket, "STAT$CRLF") or return;
-    if($stat =~ m#^\+OK (\d+) (\d+)#)
-    {   @$self{qw(MTP_messages MTP_total)} = ($1,$2);
-    }
-    else
+    if($stat !~ m#^\+OK (\d+) (\d+)#)
     {   delete $self->{MTP_messages};
         delete $self->{MTP_size};
         $self->log(ERROR => "POP3 Could not do a STAT");
         return;
     }
+    $self->{MTP_messages}    = my $nr_msgs = $1;
+    $self->{MTP_folder_size} = $2;
 
-# Check if we can do a UIDL
+    # Check if we can do a UIDL
 
     my $uidl = $self->send($socket, "UIDL$CRLF") or return;
     $self->{MTP_nouidl} = undef;
-    delete $self->{MTP_uidl2n}; # lose the reverse lookup: UIDL -> number
-    if(OK($uidl))
-    {   my @n2uidl;
-        $n2uidl[$self->{MTP_messages}] = undef; # optimization, sets right size
+    delete $self->{MTP_uidl2n}; # drop the reverse lookup: UIDL -> number
 
-        local $_;    # protect global $_
-        while(<$socket>)
-        {   last if substr($_, 0, 1) eq '.';
-            s#\r?\n$##;
-            $n2uidl[$1] = $2 if m#^(\d+) (.+)#;
+    if(_OK $uidl)
+    {   my @n2uidl;
+        $n2uidl[$nr_msgs] = undef; # pre-alloc
+
+        while(my $line = <$socket>)
+        {   last if substr($line, 0, 1) eq '.';
+            $line =~ m#^(\d+) (.+?)\r?\n# or next;
+            $n2uidl[$1] = $2;
         }
 
         shift @n2uidl; # make message 1 into index 0
@@ -354,35 +350,31 @@ sub status($;$)
         delete $self->{MTP_n2length};
         delete $self->{MTP_nouidl};
     }
-
-# We can't do UIDL, we need to fake it
-
     else
-    {   my $list = $self->send($socket, "LIST$CRLF") or return;
-        my @n2length;
-        my @n2uidl;
-        if(OK($list))
-        {   my $messages = $self->{MTP_messages};
-            my ($host, $port) = $self->remoteHost;
-            $n2length[$messages] = $n2uidl[$messages] = undef; # optimization
-            while(<$socket>)
-            {   last if substr($_, 0, 1) eq '.';
-                m#^(\d+) (\d+)#;
+    {   # We can't do UIDL, we need to fake it
+        my $list = $self->send($socket, "LIST$CRLF") or return;
+        my (@n2length, @n2uidl);
+
+        if(_OK $list)
+        {   $n2length[$nr_msgs] = $n2uidl[$nr_msgs] = undef; # alloc all
+
+            my ($host, $port)    = $self->remoteHost;
+            while(my $line = <$socket>)
+            {   last if substr($line, 0, 1) eq '.';
+                $line =~ m#^(\d+) (\d+)# or next;
                 $n2length[$1] = $2;
-                $n2uidl[$1] = "$host:$port:$1"; # fake UIDL, for id only
+                $n2uidl[$1]   = "$host:$port:$1"; # fake UIDL, for id only
             }
             shift @n2length; shift @n2uidl; # make 1st message in index 0
         }
         $self->{MTP_n2length} = \@n2length;
-        $self->{MTP_n2uidl} = \@n2uidl;
+        $self->{MTP_n2uidl}   = \@n2uidl;
     }
 
     my $i = 1;
-    my %uidl2n;
-    foreach(@{$self->{MTP_n2uidl}})
-    {   $uidl2n{$_} = $i++;
-    }
+    my %uidl2n = map +($_ => $i++), @{$self->{MTP_n2uidl}};
     $self->{MTP_uidl2n} = \%uidl2n;
+
     1;
 }
 
@@ -390,8 +382,10 @@ sub status($;$)
 
 
 sub url(;$)
-{   my ($host, $port, $user, $pwd) = shift->remoteHost;
-    "pop3://$user:$pwd\@$host:$port";
+{   my $self = shift;
+    my ($host, $port, $user, $pwd) = $self->remoteHost;
+    my $proto = $self->useSSL ? 'pop3s' : 'pop3';
+    "$proto://$user:$pwd\@$host:$port";
 }
 
 #------------------------------------------
